/*
 * CrudContainer
 * Copyright (C) 2011 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.filter.lucene;

import dk.apaq.filter.Filter;
import dk.apaq.filter.core.AndFilter;
import dk.apaq.filter.core.CompareFilter;
import dk.apaq.filter.core.LikeFilter;
import dk.apaq.filter.core.NotFilter;
import dk.apaq.filter.core.OrFilter;
import org.apache.lucene.queryParser.QueryParser;


/**
 * See http://lucene.apache.org/java/3_1_0/queryparsersyntax.html
 * @author michaelzachariassenkrog
 * @deprecated Use FilterTranslatorForLucene instead
 */

public class FiltrationLuceneTranslator {

    private static final String[] ESCAPE_CHARS = {"+", "-", "&&", "||", "!",
                                                    "(", ")", "{", "}", "[", "]",
                                                    "^", "\"", "~", ":", "\\"};
    private static final FiltrationLuceneTranslator SINGLETON = new FiltrationLuceneTranslator();
    
    public static FiltrationLuceneTranslator getTranslator() {
        return SINGLETON;
    }

    public String translate(Filter filter) {
        if(filter instanceof LikeFilter) {
            LikeFilter cf = (LikeFilter)filter;
            if(cf.getValue() == null) {
                return null;
            }

            String searchString = cf.getValue().trim();
            searchString = searchString.replace(" *", "");

            String[] valueArray = searchString.split(" ");
            StringBuilder sb = new StringBuilder();

            for(String value:valueArray) {
                value = value.trim();
                if("".equals(value)) {
                    continue;
                }

                if(value.endsWith("*")) {
                    value = value.substring(0, value.length()-1);
                }

                if("&".equals(value) || value.startsWith("-") || value.startsWith("+")) {
                    continue;
                }

                if(sb.length()>0) {
                    sb.append(" AND ");
                }

                value = QueryParser.escape(value);

                sb.append("(");
                sb.append(cf.getPropertyId());
                sb.append(":");
                sb.append(value);
                if(cf.getPrecision() < 1) {
                    sb.append("~");
                    sb.append(cf.getPrecision());
                }
                sb.append(" OR ");
                sb.append(cf.getPropertyId());
                sb.append(":");
                sb.append(value);
                sb.append("*");
                sb.append(")");
            }

            sb.insert(0, "(");
            sb.append(")");
            return sb.toString();
        }

        if(filter instanceof CompareFilter) {
            CompareFilter cf = (CompareFilter)filter;
            switch(cf.getCompareType()) {
                case Equals:
                    return cf.getPropertyId() + ":" +cf.getValue();

                    //TODO
                case GreaterOrEqual:
                    return cf.getPropertyId() + ":[" + cf.getValue() + " TO " + Long.MAX_VALUE + "]";
                case GreaterThan:
                    return cf.getPropertyId() + ":{" + cf.getValue() + " TO " + Long.MAX_VALUE + "}";
                case LessOrEqual:
                    return cf.getPropertyId() + ":[0 TO " + cf.getValue() + "]";
                case LessThan:
                    return cf.getPropertyId() + "{0 TO " + cf.getValue() + "}";
            }
        }

        if(filter instanceof AndFilter) {
            AndFilter af = (AndFilter) filter;
            StringBuilder sb = new StringBuilder();

            //sb.append("(");
            for(int i=0;i<af.getFilters().size();i++) {
                if (i>0) {
                   sb.append(" AND ");
                }
                sb.append(translate(af.getFilters().get(i)));
            }
            //sb.append(")");
            return sb.toString();
        }

        if(filter instanceof OrFilter) {
            OrFilter af = (OrFilter) filter;
            StringBuilder sb = new StringBuilder();

            //sb.append("(");
            for(int i=0;i<af.getFilters().size();i++) {
                if (i>0) {
                   sb.append(" OR ");
                }
                sb.append(translate(af.getFilters().get(i)));
            }
            //sb.append(")");
            return sb.toString();
        }

        if(filter instanceof NotFilter) {
            NotFilter af = (NotFilter) filter;
            return " NOT " + translate(af.getFilterElement());
        }

        return null;
    }

    
}
